/****************************************/
/*	Nelio Alves Pereira Filho	*/
/*	EP3 - MAC323 - NUSP: 2875125	*/
/*	Profa. Cristina Gomes		*/
/****************************************/


/* 
   Algoritmos bsicos para rvores de busca binria
*/


#include <stdio.h>
#include <malloc.h>
#include <stdarg.h>
#include "avl.h"
#include "abb.h"


celula_pt nova_celula (abb *t) {
  /* Devolve um ponteiro para uma nova clula */
  celula_pt aux;

  if (t->livre == NULL) {
    /* Se a lista livre est vazia, aloca mais uma clula */
    if ((aux = (celula_pt) malloc(sizeof(celula))) == NULL) {
      fprintf(stderr, "Erro: memoria insuficiente!\n");
      exit(0);
    }
    return(aux);
  }
  else {
    /* seno pega uma clula da lista livre */
    aux = t->livre;
    t->livre = t->livre->esq;
    return(aux);
  }
}

void libera_celula (abb *t, celula_pt p) {
  /* Insere a clula apontada por p na lista livre */
  p->esq = t->livre;
  t->livre = p;
}

void inicializa_abb (abb *t) 
{
  t->raiz = t->livre = NULL;
}



celula_pt busca_abb (abb *t, tipo_info k, func_ptr func, va_list *ptr) 
{
  /* Busca o n com chave k na rvore dada por t */
  int r;
  celula_pt p;
  
  if (k == NULL)
    printf("ERR: busca_abb, k == NULL\n");
  
  p = t->raiz;
  
  if (p == NULL)  printf("ERR: busca_abb, RAIZ  == NULL\n");
  
  // imprime(p,0);
  
  
  while ( (p != NULL) && (r = func (k, p->info, ptr)) ) {
    //   printf("    Buscando: lado = (%d)\n", r);
    
    if (r < 0){
      p = p->esq;
      //     if (p == NULL) printf("    buscando: lado esq == NULL\n");
      //  else printf("    buscano:lado esq != NULL\n");
      
    }
    else{
      p = p->dir;
      //      if (p == NULL) printf("    buscando: lado dir == NULL\n");
      //   else printf("    buscano:lado esq != NULL\n");
    }
    
  }
  if ( p == NULL)printf("ERR: busca_abb, nao encontrou\n");
  return p;
}



tipo_info before_abb (abb *a, tipo_info k, func_ptr func, va_list *ptr)
{
  celula_pt p, q;

  printf("before_abb\n");
  
  p = busca_abb (a, k, func, ptr);
 
  if (p == NULL)
    printf("before_abb: Nao encontrou k\n");
  
  if (p->esq != NULL) {
    
    for (q = p->esq; q->dir != NULL; q = q->dir);
    
    return q->info;
  }
  else {
    for (q = p->pai; (q != NULL) && (func (q->info, p->info, ptr) > 0); q = q->pai);
    
    return (q != NULL) ? q->info : NULL;
  }
}



tipo_info after_abb (abb *a, tipo_info k, func_ptr func, va_list *ptr)
{
  celula_pt p, q;
  printf("after_abb\n");

  p = busca_abb (a, k, func, ptr);
  if (p == NULL)
    printf("after_abb: Nao encontrou k\n");

  if (p->dir != NULL) {

    for (q = p->dir; q->esq != NULL; q = q->esq);
   
    return q->info;
  }
  else {
    for (q = p->pai; (q != NULL) && (func (q->info, p->info, ptr) < 0); q = q->pai);

    return (q != NULL) ? q->info : NULL;
  }
}





celula_pt insere_abb (abb *t, tipo_info x, func_ptr func, va_list *ptr) 
{
  /* Insere o elemento x na rvore dada por  t */
  celula_pt p, ant, q;
  int r;

  /* Coloca x em uma nova clula */
  q = nova_celula(t);
  q->info = x;
  q->esq = q->dir = NULL;


  /* Busca a posio de x na rvore */
  ant = NULL;
  p = t->raiz;

  while ((p != NULL) && (r=func(x, p->info, ptr))) {
    ant = p;
    if (r < 0)
      p = p->esq;
    else
      p = p->dir;
  }

  /* Insere x na posio dada pela busca */
  if ((ant == NULL))// && r) //jm -  mudei aqui
    t->raiz = q;
  else if (r < 0)
    ant->esq = q;
  else
    ant->dir = q; 

  q->pai = ant;
  return q;
}


celula_pt minimo (celula_pt r) 
{
  /* Devolve a clula de menor valor na rvore com raiz r */
  celula_pt p, ant;
  
  p = r;
  ant = NULL;
  while (p != NULL) {
    ant = p;
    p = p->esq;
  }

  return ant;
}


int delete_abb (abb *t, tipo_info k, func_ptr func, va_list *ptr) 
{
  /* Deleta da rvore dada por t o elemento de chave k */
  celula_pt q, p, f;

  q = busca_abb (t, k, func, ptr);
  
  if (q == NULL) {
    return 0;
  }
  else {
    /* Determina a clula que ser efetivamente deletada da rvore */
    if ( (q->esq == NULL) || (q->dir == NULL) ) {
      p = q;
    }
    else {
      p = minimo(q->dir);
      q->info = p->info;
    }
    /* p tem no mximo um filho */ 
    if (p->esq != NULL)
      f = p->esq;
    else 
      f = p->dir;
    /* f  NULL se p no tem filhos, ou o seu nico filho */

    /* Remove p da rvore */
    if (f != NULL) {
      f->pai = p->pai;
    }
    if (p->pai == NULL) {
      t->raiz = f;
    }
    else if (p->pai->esq == p) {
      p->pai->esq = f;
    }
    else {
      p->pai->dir = f;
    }
    libera_celula(t, p);

    return 1;
  }
}




int delete_cel_abb (abb *t, celula_pt q) 
{
  /* Deleta da rvore dada por t o elemento de chave k */
  celula_pt p, f;

  if (q == NULL) {
    return 0;
  }
  else {
    /* Determina a clula que ser efetivamente deletada da rvore */
    if (q->esq == NULL || q->dir == NULL) 
      p = q;
    else {
      p = minimo(q->dir);
      q->info = p->info;
    }

    /* p tem no mximo um filho */ 
    if (p->esq != NULL)
      f = p->esq;
    else 
      f = p->dir;
    /* f  NULL se p no tem filhos, ou o seu nico filho */
    
    /* Remove p da rvore */
    if (f != NULL) 
      f->pai = p->pai;
    if (p->pai == NULL)
      t->raiz = f;
    else if (p->pai->esq == p)
      p->pai->esq = f;
    else
      p->pai->dir = f;
    libera_celula(t, p);

    return 1;
  }
}













