/****************************************/
/*	Nelio Alves Pereira Filho	*/
/*	EP3 - MAC323 - NUSP: 2875125	*/
/*	Profa. Cristina Gomes		*/
/****************************************/


#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>
#include "segmento.h"
#include "avl.h"
#include "abb.h"



int member (abb *a, tipo_info k, func_ptr func, ...) 
{
  va_list ptr;
  int res;
  
  va_start (ptr, func);
  res = (busca_abb(a, k, func, &ptr) != NULL);
  va_end (ptr);

  reset();

  return res;
}




/** Rotaciona a arvore para a esquerda em p **/

void RotacionaEsquerda (abb *t, celula_pt p)
{
  celula old = *p;
  
  if (p->dir != NULL)
  {
    if (p->pai == NULL)
      t->raiz = p->dir;
    else
    {
      if (p->pai->esq == p)	/* filho esq */
        p->pai->esq = p->dir;
      else
        p->pai->dir = p->dir;
    }
    p->dir = old.dir->esq;
    if (old.dir->esq != NULL)
      old.dir->esq->pai = p;
    old.dir->pai = old.pai;
    old.dir->esq = p;
    p->pai = old.dir;
  }
  else
  {
    fprintf(stderr, "Erro: nao pode rotacionar para esquerda em %p!\n",p->info);
    exit(0);    
  }
}


/** Rotaciona a arvore para a direita em p **/

void RotacionaDireita (abb *t, celula_pt p)
{
  celula old = *p;
  
  if (p->esq != NULL)
  {
    if (p->pai == NULL)
      t->raiz = p->esq;
    else
    {
      if (p->pai->esq == p)	/* filho esq */
        p->pai->esq = p->esq;
      else
        p->pai->dir = p->esq;
    }
    p->esq = old.esq->dir;
    if (old.esq->dir != NULL)
      old.esq->dir->pai = p;
    old.esq->pai = old.pai;
    old.esq->dir = p;
    p->pai = old.esq;
  }
  else
  {
    fprintf(stderr, "Erro: nao pode rotacionar para direita em %p!\n",p->info);
    exit(0);    
  }
}



void balance (abb *a, celula_pt q)
{
  int d = altura(q->dir)-altura(q->esq);

  if ( abs(d) > 1 )
  {
    if (d > 0)
    {
      if (q->dir->esq != NULL)
        RotacionaDireita(a, q->dir);
      RotacionaEsquerda(a, q);
    }
    else
    {
      if (q->esq->dir != NULL)
        RotacionaEsquerda(a, q->esq);
      RotacionaDireita(a, q);
    }
  }
}


void insere_avl (abb *a, tipo_info p, func_ptr func, ...) 
{
  celula_pt q;		// a celula onde foi inserida
  va_list ptr;
  
  va_start (ptr, func);

  q = insere_abb (a, p, func, &ptr);
  
  if (q == NULL)
    printf("ERR: insere_avl, q == NULL\n");
      
  /* Rearranja, se necessario */
  
  //q = busca_abb (a, p, func, &ptr);
  while (q != NULL) {
    balance(a, q);
    q = q->pai;
  }

  va_end (ptr);

  reset();
}



void delete_avl (abb *a, tipo_info k, func_ptr func, ...) 
{
  celula_pt q;
  va_list ptr;

  va_start (ptr, func);
  if (k == NULL)
    printf("ERR: delete_avl, k == NULL\n");
  
  q = busca_abb (a, k, func, &ptr);
  
  if (q == NULL){
    printf("ERR: delete_avl, q == NULL ( NAO Encontrou k (%d))\n", ((segmento *)k)->id);
    //  return;
  }
  
  delete_abb (a, k, func, &ptr);

  /* rearranja se necessario */
  //if (a->raiz == NULL) return;

  q = q->pai;
  while (q != NULL)
  {
    balance(a, q);
    q = q->pai; 
  }  

  va_end (ptr);

  reset();
}


void delete_cel_avl (abb *a, celula_pt q) 
{
  delete_cel_abb (a, q);
  
  /* rearranja se necessario */
  //if (a->raiz == NULL) return;
  
  q = q->pai;
  while (q != NULL)
  {
    balance(a, q);
    q = q->pai; 
  } 
  
}


/** altura de um no **/

int altura (celula_pt p)
{
  int esq, dir;

  if (p == NULL){
    //printf ("NULL\n");
    return 0;
  }
  
  else {
    esq = altura(p->esq);
    dir = altura(p->dir);
    
    if (esq > dir)
      return (1 + esq);
    else
      return (1 + dir);
  }
}


/* retorna o cara anterior a k na ordem estabelecida */

tipo_info before_avl (abb *a, tipo_info k, func_ptr func, ...)
{
  va_list ptr;
  tipo_info ret;
  
  va_start (ptr, func);
  ret = before_abb (a, k, func, &ptr);
  va_end (ptr);

  reset();
  
  return ret;
}



/* retorna o cara anterior a k na ordem estabelecida */

tipo_info after_avl (abb *a, tipo_info k, func_ptr func, ...)
{
  va_list ptr;
  tipo_info ret;
  
  va_start (ptr, func);
  ret = after_abb (a, k, func, &ptr);
  va_end (ptr);

  reset();
  
  return ret;
}







