#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <unistd.h>
#include <malloc.h>
#include <math.h>
#include <gtk/gtk.h>

#include <glib.h>
// Open GL
#include <GL/gl.h>
#include <GL/glu.h>

// glarea
#include <gtkgl/gtkglarea.h>

// WINDOW stuff
#include "window.h"

#include "segmento.h"
#include "intersec.h"

#include "avl.h"

void glarea_redraw();
gint glarea_expose_cb(GtkWidget *glarea, GdkEventExpose *event, gpointer data);
gint glarea_motion_notify_cb(GtkWidget *glarea, GdkEventAny *event, gpointer data);
gint glarea_configure_cb(GtkWidget *glarea, GdkEventConfigure *event, gpointer data);


//------------------------------------------------------------
// Variaveis globais
//------------------------------------------------------------
GCond*   cond = NULL;   /* Must be initialized somewhere */
GMutex*  mutex = NULL; /* Must be initialized somewhere */
int go  = FALSE;


// area de desenho
GtkWidget *glarea;

extern abb *abb_eventpoints;

extern GList *segmentos;
extern GList *pontos;
extern GList *interseccoes;

extern point *pontoevento;


// segmento temporario (qdo desenhando)
static int clicked = FALSE;
static segmento click_segment;

//point *intersec;

float COR_SWEEPLINE[] = {0.4, 0.8, 0.6};
float COR_SWEEPLINE_SEG[] = {0.5, 1.0, 0.5};

float COR_INTERSEC[] = {0.8,0.3,0.3};
float colors[][3]=  /* saturation of 0.5, hue steps of 45 */
{ {1.00, 0.50, 0.50},
  {1.00, 0.88, 0.50},
  {0.75, 1.00, 0.50},
  {0.50, 1.00, 0.62},
  {0.50, 1.00, 1.00},
  {0.50, 0.62, 1.00},
  {0.75, 0.50, 1.00},
  {1.00, 0.50, 0.88} };

//-------------------------------------------------------------------
// Calcula a interseccao dos segmentos em 'segments'
//-------------------------------------------------------------------
#define SMALL 1E-3

int loadSegmentos_cb(GtkWidget *entry)
{
  readfile(gtk_entry_get_text(GTK_ENTRY( entry )));
  glarea_redraw();
  return TRUE;
}

int saveSegmentos_cb(GtkWidget *entry)
{
  savefile(gtk_entry_get_text(GTK_ENTRY( entry )));
  glarea_redraw();
  return TRUE;
}



int interseccao(GtkWidget *w, gpointer data)
{
  int algoritmo = (int)data;
  
  int numSegments = g_list_length(segmentos);
  
  printf("n Segmentos: %d\n", numSegments);
 
  // precisa da free em todos antes !!!
  g_list_free(interseccoes);
  interseccoes = NULL;
  
  switch (algoritmo){

  case INGENUO:
    ingenuo(&segmentos);
    break;
  
  case SMART:
    smart(&pontos);
    break;
    
  
  default:
    printf("Algoritmo nao implementado!\n");
    return FALSE;
  }
  glarea_redraw();	  
  
  return TRUE;
  
}
void drawSweepline ()
{
  int h = glarea->allocation.height;
  
  glColor3fv( COR_SWEEPLINE );
  glBegin(GL_LINES);
  glVertex2f (pontoevento->x, 0);
  glVertex2f (pontoevento->x, h);
  //drawSweepline_aux(r);
  glEnd();

}

//-------------------------------------------------------------------
// Manipula as interseccoes
//-------------------------------------------------------------------

void drawPoint (gpointer data, gpointer userdata)
{
  point *ponto = (point *)data;
  
  glColor3fv( COR_INTERSEC );
  glBegin(GL_LINE_STRIP);
  glVertex2f (ponto->x - TAM_PONTO, ponto->y - TAM_PONTO);
  glVertex2f (ponto->x + TAM_PONTO, ponto->y - TAM_PONTO);
  glVertex2f (ponto->x + TAM_PONTO, ponto->y + TAM_PONTO);
  glVertex2f (ponto->x - TAM_PONTO, ponto->y + TAM_PONTO);
  glVertex2f (ponto->x - TAM_PONTO, ponto->y - TAM_PONTO);
  glEnd();
  //  glRectf( ponto->x - TAM_PONTO,  ponto->y - TAM_PONTO,  
  //	   ponto->x + TAM_PONTO,  ponto->y + TAM_PONTO);
  
}

void drawEventPoints (celula_pt r) {
  point *ponto;
   
  if (r != NULL) { 
    ponto = (point  *)r->info;
       
    glColor3f( 0.3,0.6,0.4 );
    glBegin(GL_LINE_STRIP);
    glVertex2f (ponto->x - 2, ponto->y - 2);
    glVertex2f (ponto->x + 2, ponto->y - 2);
    glVertex2f (ponto->x + 2, ponto->y + 2);
    glVertex2f (ponto->x - 2, ponto->y + 2);
    glVertex2f (ponto->x - 2, ponto->y - 2);
    glEnd();
    
    drawEventPoints (r->esq); 
    drawEventPoints (r->dir);
  }
}


//-------------------------------------------------------------------
// Manipula os segmentos
//-------------------------------------------------------------------
void drawSegment(gpointer data, gpointer userdata)
{
  segmento *segment = (segmento *)data;
  if (segment == NULL) return;
  
  glColor3fv(segment->cor);
  glVertex2f(segment->esq->x, segment->esq->y);
  glVertex2f(segment->dir->x, segment->dir->y);  
}





//-------------------------------------------------------------------
// Parte de desenho
//-------------------------------------------------------------------


void drawGrid(GtkWidget *glarea)
{
  int num = 10;
  
  int i;
  int w = glarea->allocation.width;
  int h = glarea->allocation.height;
  
  float delta =  w / num;
  
  glColor3f(0.2,0.2,0.25);
  
  glBegin (GL_LINES);
  i = 0;
  
  while (i < w)
  {
    glVertex3f(i, 0, -0.1);
    glVertex3f(i, h, -0.1);      
    i += delta;
  }
  
  i = 0;
  while ( i < h )
  {
    glVertex3f(0, i, -0.1);
    glVertex3f(w, i, -0.1);      
    i +=  delta;
  }
 
  glEnd();
  
}

   
//-------------------------------------------------------------------
// Redraw Screen i.e will call expose_cb
//-------------------------------------------------------------------
void glarea_redraw()
{
  GdkRectangle area;
  area.x = 0;
  area.y = 0;
  area.width  = glarea->allocation.width;
  area.height = glarea->allocation.height;
  gtk_widget_draw(glarea, &area);
}



/* When widget is exposed it's contents are redrawn. */
gint glarea_expose_cb(GtkWidget *glarea, GdkEventExpose *event, gpointer data)
{
  /* Draw only last expose. */
  if (event->count > 0) return TRUE;
  
  
  /* OpenGL functions can be called only if make_current returns true */
  if (!gtk_gl_area_make_current( GTK_GL_AREA(glarea) ))
    return FALSE;
  
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);      

  drawGrid(glarea);
    
  glBegin(GL_LINES);

  if (clicked)
    drawSegment((gpointer)&click_segment, NULL);
  
  g_list_foreach(segmentos, drawSegment, NULL);
  
  glEnd();    	

  g_list_foreach(interseccoes, drawPoint, NULL);

  if (pontoevento != NULL){
    //    printf("*** SWEEPLINE: %f\n",pontoevento->x);
    drawSweepline();
  }
  
  if (abb_eventpoints != NULL){
    drawEventPoints(abb_eventpoints->raiz);
  }
  
  
    
  {
    char text[10];
    sprintf(text, "%d",g_list_length(segmentos));
    gtk_label_set_text(GTK_LABEL( data ), text);
  }      
  
  
  /* Swap backbuffer to front */
  gtk_gl_area_swapbuffers ( GTK_GL_AREA(glarea) );


  return TRUE;
}




gint glarea_motion_notify_cb(GtkWidget *glarea, GdkEventAny *event, gpointer data)
{
  
  switch (event->type) 
    {
      
    case GDK_BUTTON_RELEASE:
      {
	//	GdkEventButton *eventbutton = (GdkEventButton *)event;

	if (clicked){
	  //	  printf("RELEASE: x: (%f, %f)\n",eventbutton->x, eventbutton->y);
	  if ((click_segment.esq->x != click_segment.dir->x) ||
	      (click_segment.esq->y != click_segment.dir->y))
	    {
	      addSegment(click_segment.esq->x, click_segment.esq->y,
			 click_segment.dir->x, click_segment.dir->y);
	    }
	  
	  clicked = FALSE;
	      
	  glarea_redraw();
	  return TRUE;
	}
	return FALSE;
	
	  
      }
      
    case GDK_BUTTON_PRESS:
      {
	GdkEventButton *eventbutton = (GdkEventButton *)event;
	GdkModifierType state; 

	state = eventbutton->state;
	
	if (eventbutton->button == 1){
	  clicked = TRUE;
	  click_segment.esq->x = (int)eventbutton->x;
	  click_segment.esq->y = (int)eventbutton->y;
	  click_segment.dir->x = (int)eventbutton->x;
	  click_segment.dir->y = (int)eventbutton->y;
	  
	  //	  printf("PRESS  : x: (%f, %f)\n",eventbutton->x,  eventbutton->y);

	  glarea_redraw();
	
	  return TRUE;
	}
	
	return FALSE;
      }
      
      
    case GDK_MOTION_NOTIFY:  {
      int x, y;
      
      GdkModifierType state; 
      GdkEventMotion *eventmotion = (GdkEventMotion *)event;
      
      if (eventmotion->is_hint) 
	gdk_window_get_pointer(eventmotion->window, &x, &y, &state);
      else {
	x = eventmotion->x;
	y = eventmotion->y;
	state = eventmotion->state;
      }      

      if (clicked){
	//	printf("motion: (%d, %d)\n",x,y);
	click_segment.dir->x = (int)x;
	click_segment.dir->y = (int)y;

	glarea_redraw();
      }
      
      return TRUE;
    }
    
    default:
      return FALSE;
    }
  
    return TRUE;
    
}




/* When glarea widget size changes, viewport size is set to match the new size */
gint glarea_configure_cb(GtkWidget *glarea, GdkEventConfigure *event, gpointer data)
{
  
  /* OpenGL functions can be called only if make_current returns true */
  if (!gtk_gl_area_make_current(GTK_GL_AREA(glarea)))
    return FALSE;
  
  glViewport(0,0, glarea->allocation.width, glarea->allocation.height);
  
  // ------------------------------
  // Configura a matriz de projecao 
  // ------------------------------
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  
  //  gluOrtho2D(0, glarea->allocation.width, glarea->allocation.height, 0);
   glOrtho(0, glarea->allocation.width, glarea->allocation.height, 0,  -1, 1);
  
  // ------------------------------
  // Configura a matriz modelview 
  // ------------------------------
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
    
  return TRUE;
}



gint glarea_init_cb(GtkWidget *glarea, gpointer data)
{
  
  glClearColor (0.1, 0.1, 0.1, 1.0);

  glEnable(GL_DEPTH_TEST);      
       
  return TRUE;
}


int nextstep(GtkWidget *w)
{
  //-------- NOTIFY ------------
  g_mutex_lock (mutex);
  printf("MUTEX LOCKED (n)\n");
  go = TRUE;
  g_cond_signal (cond);
  printf("SIGNAL COND (n)!\n");
  g_mutex_unlock (mutex);
  printf("MUTEX UNLOCKED (n)\n");
  return TRUE;
}




GtkWidget *new_window(int width, int height)
{
  GtkWidget *glArea;
  GtkWidget *window;
  
  /* Check if OpenGL is supported. */
  if (gdk_gl_query() == FALSE) {
    printf("OpenGL not supported\n");
    return FALSE;
  }

  window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
  
  gtk_window_set_title(GTK_WINDOW(window), " GEOCOMP: Intersecacao de Segmentos " );
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);
  gtk_window_set_position( GTK_WINDOW(window), GTK_WIN_POS_MOUSE );
  gtk_signal_connect (GTK_OBJECT(window), "delete-event",
		      GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

  
    
    // CRIA A PARTE DA  DISTRIBUICAO DA JANELA
  {
    GtkWidget *vbox;  
    
    vbox = add_vbox(window, FALSE, 0);
    
    {
      GtkWidget *hbox;
      GtkWidget *hbox2;

      GtkWidget *button;
      GtkWidget *label;
      GtkWidget *entry;
      
      
      hbox   = add_hbox(vbox, FALSE, 0);
      hbox2   = add_hbox(vbox, FALSE, 0);

      /* create new OpenGL widget */
      glArea = add_gl_area(vbox);


      button = new_button(hbox, "Alg. INGENUO ", interseccao, (gpointer)INGENUO);
      button = new_button(hbox, "Alg. Varredura ", interseccao, (gpointer)SMART);

      //      button = new_button(hbox, "Next Step ", nextstep, NULL);

      label  = new_mostrador(hbox, "Segmentos: ","0");

      button = new_button(hbox, " Del last ", delLastSegment, NULL);

      gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
				GTK_SIGNAL_FUNC(glarea_redraw), 
				GTK_OBJECT(glArea));



      entry  = new_entry(hbox2,"", "",  "activate", NULL, NULL);
      button = new_button(hbox2, " Save ", NULL, NULL);
      gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
				GTK_SIGNAL_FUNC(saveSegmentos_cb),
				GTK_OBJECT(entry));
 

      entry  = new_entry(hbox2,"", "",  "activate", NULL, NULL);
            button = new_button(hbox2, " Load ", NULL, NULL);
      gtk_signal_connect_object(GTK_OBJECT(button), "clicked", 
				GTK_SIGNAL_FUNC(loadSegmentos_cb), 
				GTK_OBJECT(entry));


      gtk_signal_connect (GTK_OBJECT(glArea),  "expose_event",
			  GTK_SIGNAL_FUNC(glarea_expose_cb), label);

      gtk_signal_connect (GTK_OBJECT(glArea),  "configure_event",
			  GTK_SIGNAL_FUNC(glarea_configure_cb), NULL);
      
      gtk_signal_connect (GTK_OBJECT(glArea),  "realize",
			  GTK_SIGNAL_FUNC(glarea_init_cb), NULL);
      
      gtk_signal_connect (GTK_OBJECT(glArea),  "button_press_event",
			  GTK_SIGNAL_FUNC(glarea_motion_notify_cb), NULL);
      
      gtk_signal_connect (GTK_OBJECT(glArea),  "button_release_event",
			  GTK_SIGNAL_FUNC(glarea_motion_notify_cb), NULL);

      gtk_signal_connect (GTK_OBJECT(glArea),  "motion_notify_event",
			  GTK_SIGNAL_FUNC(glarea_motion_notify_cb), NULL);
      
  
      gtk_widget_set_usize(glArea,width, height); /* minimum size */
    }
  }

  
  gtk_widget_show(window);

  glarea = glArea;
  
  return window;
}


int main (int argc, char **argv)
{
  pid_t pid;

  pid = fork();

  if (pid > 0) {
    return EX_OK;
  }

  printf( "GEOCOMP: Interseccao de segmentos\n");

  gtk_init (&argc, &argv);

  click_segment.esq = (point *)malloc(sizeof(point));
  click_segment.dir = (point *)malloc(sizeof(point));
  click_segment.cor = colors[COR_DRAG];

#if 0   
  g_thread_init(NULL);
  mutex = g_mutex_new();
  printf("CRIOU MUTEX\n");
  cond  = g_cond_new();
  printf("CRIOU COND\n");
#endif
  
  
  // ---  cria janela e mostra na tela ---
  new_window(400,400);
  
  gtk_main ();   
  return 0;
}





















