#include <glib.h>
#include <malloc.h>
#include <stdio.h>
#include <sysexits.h>

#include "segmento.h"

extern GList *segmentos;
extern GList *pontos;
extern float colors[][3];


static double u = G_MINDOUBLE;

void reset()
{
  u = G_MINDOUBLE;
}


/* devolve um novo ponto ou um ja alocado */
point* getponto (GList **Tpontos, double x, double y)
{
  GList *atual = g_list_first(*Tpontos);
  point *p;
  
  while (atual != NULL) {
    p = (point *)atual->data;
    if ((p->x == x) && (p->y == y)) { return p; }
    
    atual = g_list_next(atual);
  }
  
  if ( (p = (point *)malloc (sizeof(point))) == NULL ) {
    fprintf (stderr, "Malloc error! %d,%s\n", __LINE__, __FILE__);
    exit (EX_OSERR);
  }
  p->x = x;
  p->y = y;
  p->C = NULL;
  p->D = NULL;
  p->E = NULL;
  
  *Tpontos = g_list_append(*Tpontos, (gpointer)p);

  return p;
}

int addSegment (double x1, double y1, double x2, double y2) 
{
  static int id = 1;
  point *p1, *p2;
  segmento *res;
  
  p1 = getponto (&pontos, x1, y1);
  p2 = getponto (&pontos, x2, y2);
    
  if ( (res = (segmento *)malloc(sizeof(segmento))) == NULL ) {
    fprintf (stderr, "Erro ao alocar memoria.");
    exit (EX_OSERR);
  }
  if ((x1 < x2) || ((x1 == x2) && (y1 < y2))) {
    res->esq = p1;
    res->dir = p2;
    p1->E = g_list_append(p1->E, (gpointer)res);
    p2->D = g_list_append(p2->D, (gpointer)res);
  }
  else {
    res->esq = p2;
    res->dir = p1;
    p1->D = g_list_append(p1->D, (gpointer)res);
    p2->E = g_list_append(p2->E, (gpointer)res);

  }

  res->a     = res->dir->y - res->esq->y;
  res->denom = res->dir->x - res->esq->x;
  res->b     = res->esq->y*res->denom - res->esq->x*res->a;
  //    printf ("A %2.2f B %2.2f D %2.2f\n", res->a, res->b, res->denom);
  
  res->cor = colors[COR_NORMAL];
  res->id = id;
  
  printf("adicionou segmento: (%f,%f) (%f,%f)\n",
	 res->esq->x, res->esq->y, res->dir->x, res->dir->y);
  
  segmentos = g_list_append(segmentos, (gpointer)res);
  
  id++;
  return TRUE;
}

// sei que esta funcao deveria receber GtkWidget *w, mas e dai ?
int delLastSegment ()
{
  segmento *last;
  int size;
  
  
  if (segmentos == NULL)
    return FALSE;
  
  last = (segmento *)g_list_last(segmentos)->data;

  if (last == NULL) return FALSE;
  

  last->esq->E = g_list_remove(last->esq->E, last);
  last->dir->D = g_list_remove(last->dir->D, last);
  
  size = ( g_list_length ( last->esq->D) + g_list_length ( last->esq->E));
  //printf("SIZE AFTER : %d\n",size);
  if (size == 0)
    {
      //  printf("removendo ponto ESQ!\n");
      pontos = g_list_remove(pontos, (gpointer)last->esq);
      g_list_free(last->esq->C);
      g_list_free(last->esq->D);
      g_list_free(last->esq->E);
      free(last->esq);
    }
  
 
  size = ( g_list_length ( last->dir->D) +  g_list_length ( last->dir->E));
  // printf("SIZE after: %d\n",size);
  
  if (size == 0)
    {
      //  printf("removendo ponto DIR!\n");
      pontos = g_list_remove(pontos, (gpointer)last->dir);
      g_list_free(last->dir->C);
      g_list_free(last->dir->D);
      g_list_free(last->dir->E);
      free(last->dir);
    }

  
  segmentos = g_list_remove(segmentos, (gpointer)last);
  free(last);
  
  return TRUE;
}


/* decide se os segmentos a e b se intersectam. Se sim, p eh o ponto */

int intersec (segmento *a, segmento *b, point *p)
{
  double s, t;                 /* parametros da equacao */
  double num, denom;           /* numerador e denominador */
  
  denom = (a->esq->x * (b->dir->y - b->esq->y) + a->dir->x * (b->esq->y - b->dir->y) +
	   b->dir->x * (a->dir->y - a->esq->y) + b->esq->x * (a->esq->y - a->dir->y));
  

  /* se denom = 0 os segmentos sao paralelos */
  if (denom == 0.0) return 0;

  num = (a->esq->x * (b->dir->y - b->esq->y) + b->esq->x * (a->esq->y - b->dir->y) +
	 b->dir->x * (b->esq->y - a->esq->y));
  s = num / denom;
  
  num = - (a->esq->x * (b->esq->y - a->dir->y) + a->dir->x * (a->esq->y - b->esq->y) +
	   b->esq->x * (a->dir->y - a->esq->y));
  t = num / denom;
  

  if ((s >= 0.0) && (s <= 1.0) && (t >= 0.0) && (t <= 1.0)) {
    p->x = a->esq->x + s * (a->dir->x - a->esq->x);
    p->y = a->esq->y + s * (a->dir->y - a->esq->y);

    return 1;
  }
  else
    return 0;
  
}


/* 
   funcao que estabelece a ordem entre dois pontos 
*/

int point_func (point *a, point *b)
{
  if (a == b) return 0;
  
  if (a->x < b->x) return -1;

  if ((a->x == b->x) && (a->y < b->y)) return -1;
  
  if ((a->x == b->x) && (a->y == b->y)) return 0;

  return 1;
}


/* 
   funcao que estabelece a ordem entre dois segmentos, segundo
   abcissa x 
*/

int seg_func (segmento *a, segmento *b, va_list *ptr)
{
  
  double y1, y2;
  int ret;
  

  if (u == G_MINDOUBLE) {
    u = va_arg (*ptr, double);
  }
    
  //  printf (" SEG_FUNC a=(%d) b=(%d): U %f\n",a->id, b->id, u);
  
  if (a == NULL || b == NULL)
    printf (" ERR: SEG_FUNC: a ou b == NULL\n");
  
  //--teste--
  if (a->id == b->id) {
    //  printf(" SEG_FUNC ACHOU MESMO SEGMENTO\n");
    return 0;
  }
  
  //---------
  
  // ta errado? e se encontrar o segmento!
  if ( (a->denom == 0) && (b->denom == 0))
    return 0;
  
  /* segmentos verticais devem vir depois de todos! */
   if ( a->denom == 0 ) { return 1; }
   if ( b->denom == 0 ) { return -1; }
    
  /* intersecao de x=u com o segmento a e b */
  y1 = (b->a*u + b->b)/b->denom;
  y2 = (a->a*u + a->b)/a->denom;
  

  /* nem sera verificado se esta no segmento (suposicao) */
  
  //teste
  if (y1 == y2) {
    ret = 0;  // mudei aqui
    // printf(" seg_func: (%d) == (%d)\n",a->id,b->id);
  }
  if (y1 < y2) {
    ret = -1;
    //printf(" seg_func: (%d) < (%d)\n",a->id,b->id);
  }
  if (y1 > y2) {
    //printf(" seg_func: (%d) > (%d)\n",a->id,b->id);
    ret = 1;
  }
  
  //printf ("Seg (%2.2f,%2.2f) (%2.2f,%2.2f) %f ? %d\n", a->esq->x, a->esq->y,
  //  b->esq->x, b->esq->y, u, ret);
  

  return ret;
  
}



