/*  *** 8kcab.fly.br:/home/jm/um/window.c,
    Tue Jul 11 12:13:46 2000, edit by jm ***  */

#include <stdio.h>
#include <stdlib.h>

// gtk
#include <gtk/gtk.h>

// opengl
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>

#include "window.h"

// HELP:
// -----
// parent, item to add, expand, fill, padding
// gtk_box_pack_start(GTK_BOX(parent), menubar, FALSE, TRUE, 0);


//------------------------------------------------------------------------
// Adiciona um StatusBar em parent
//------------------------------------------------------------------------
GtkWidget *add_status_bar(GtkWidget *parent)
{
  GtkWidget *statusbar;
  
  statusbar = gtk_statusbar_new();
  if (parent != NULL){
    if (GTK_IS_BOX(parent))
      gtk_box_pack_start( GTK_BOX(parent), statusbar, FALSE, FALSE, 0 );
    else
      gtk_container_add( GTK_CONTAINER(parent), statusbar );
  }
  gtk_widget_show(statusbar);
  return (statusbar);
}




//------------------------------------------------------------------------
// Cria um hbox e o adiciona em parent
//------------------------------------------------------------------------
GtkWidget *add_hbox( GtkWidget *parent, int homog, int spacing )
{
  GtkWidget *hbox;
  
  hbox = gtk_hbox_new( homog, spacing );
  gtk_container_set_border_width( GTK_CONTAINER(hbox), spacing );
  
  if (parent != NULL){
    if (GTK_IS_BOX(parent))
      gtk_box_pack_start( GTK_BOX(parent), hbox, FALSE, FALSE, 0 );
    else
      gtk_container_add( GTK_CONTAINER(parent), hbox );
  }
  
  gtk_widget_show( hbox );
  
  return hbox;
}

//------------------------------------------------------------------------
// Cria um vbox e o adiciona em parent
//------------------------------------------------------------------------
GtkWidget *add_vbox( GtkWidget *parent, int homog, int spacing )
{
  GtkWidget *vbox;
  
  vbox = gtk_vbox_new( homog, spacing );
  gtk_container_set_border_width( GTK_CONTAINER(vbox), spacing );
  
  if (parent != NULL){
    if (GTK_IS_BOX(parent))
      gtk_box_pack_start( GTK_BOX(parent), vbox, FALSE, FALSE, 0 );
    else
      gtk_container_add( GTK_CONTAINER(parent), vbox );
  }
  gtk_widget_show( vbox );
  return vbox;
}


//------------------------------------------------------------------------
// Adiciana um separador.
// * OBS: parent pode ser tanto um MENU quanto um BOX 
//------------------------------------------------------------------------
void add_separator( GtkWidget *parent )
{
  GtkWidget *separator;
  
  if (GTK_IS_MENU(parent)) {
    separator = gtk_menu_item_new( );
    gtk_menu_append( GTK_MENU(parent), separator );
  }
  else {
    separator = gtk_hseparator_new( );
    gtk_box_pack_start( GTK_BOX(parent), separator, TRUE, TRUE, 0 );
  }
  gtk_widget_show( separator );
}




//------------------------------------------------------------------------
//  Cria um botao com nome e adiciona em box_parent 
//  Conecta funcao ao signal clicked do botao
//------------------------------------------------------------------------
GtkWidget *new_button(GtkWidget *box_parent, const char *nome, void *funcao, 
		      gpointer data)
{
  GtkWidget *button;
 if (nome == NULL)
   button = gtk_button_new ();    
 else
   button = gtk_button_new_with_label (nome);    
  if (funcao != NULL)
    gtk_signal_connect( GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(funcao), data );
  
  gtk_box_pack_start(GTK_BOX(box_parent), button, FALSE, TRUE, 1);
  gtk_widget_show(button);
  return button;
}

//------------------------------------------------------------------------
// Cria um check-button com nome, data e adiciona em box_parent 
//  Conecta funcao ao signal clicked do botao
//------------------------------------------------------------------------
GtkWidget *new_check_button(GtkWidget *box_parent, char *nome, gint status, 
			    void *funcao, gpointer data)
{
  GtkWidget *cbutton;
  cbutton = gtk_check_button_new_with_label (nome);    
  if (funcao != NULL)
    gtk_signal_connect (GTK_OBJECT (cbutton), "clicked", 
			GTK_SIGNAL_FUNC(funcao), data);
  GTK_TOGGLE_BUTTON(cbutton)->active = status;
  gtk_box_pack_start(GTK_BOX(box_parent), cbutton, TRUE, TRUE, 0);
  gtk_widget_show(cbutton);
  return cbutton;
}

//-----------------------------------------------------------------------
// Equivalente a CheckButtons, mas so um pode estar ligado.
// ATENCAO: Para iniciar grupo ->  other == NULL
//-----------------------------------------------------------------------
GtkWidget *new_radio_button(GtkWidget *parent, GtkWidget *other, gchar *nome, 
			    gint status, void *funcao, gpointer data)
{
  GtkWidget *button = NULL;
  
  if (other == NULL) // cria group
    button = gtk_radio_button_new_with_label (NULL, nome);
  else
    button = gtk_radio_button_new_with_label(					     gtk_radio_button_group (GTK_RADIO_BUTTON (other)), nome);
  
  if (GTK_IS_BOX(parent))
    gtk_box_pack_start (GTK_BOX (parent), button, TRUE, TRUE, 0);
  
  if (funcao != NULL)
    gtk_signal_connect (GTK_OBJECT (button), "toggled", 
			GTK_SIGNAL_FUNC(funcao), data);
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), status);
  //GTK_TOGGLE_BUTTON(button)->active = status;
    
    
  gtk_widget_show (button);
  return button;
}




GtkWidget *
add_label( GtkWidget *parent_w, const char *label_text )
{
  GtkWidget *label_w;
  GtkWidget *hbox_w;
  
  label_w = gtk_label_new( label_text );
  if (GTK_IS_BOX(parent_w))
    gtk_box_pack_start( GTK_BOX(parent_w), label_w, FALSE, FALSE, 0 );
  else if (GTK_IS_BUTTON(parent_w)) {
    /* Labels are often too snug inside buttons */
    hbox_w = add_hbox( parent_w, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX(hbox_w), label_w, TRUE, FALSE, 5 );
  }
  else
    gtk_container_add( GTK_CONTAINER(parent_w), label_w );
  gtk_widget_show( label_w );
  
  return label_w;
}


//------------------------------------------------------------------------
// Cria regiao de entrada e um label (nome) e adiciona no parent  box!, 
// Conecta a funcao callback a evento, passando data como parametro 
//
//------------------------------------------------------------------------
GtkWidget *new_entry(GtkWidget *box,  gchar *prenome, gchar *posnome,  gchar *evento, 
		     void *funcao, gpointer data)
{
  
  GtkWidget *entry, *hbox, *l_nome;
  hbox = gtk_hbox_new(FALSE, 0);
  entry = gtk_entry_new();

  if (posnome != NULL){
    l_nome = gtk_label_new(posnome);
    gtk_box_pack_end(GTK_BOX(hbox), l_nome, FALSE, TRUE, 2);
    gtk_widget_show(l_nome);
  }

  /* handle da funcao */
  if (funcao != NULL)
    gtk_signal_connect (GTK_OBJECT (entry), evento,
			GTK_SIGNAL_FUNC (funcao), data );
  gtk_widget_set_usize(entry, 100, -1);     // TAMANHO
  gtk_box_pack_end(GTK_BOX(hbox), entry, FALSE, TRUE, 0);
  gtk_widget_show(entry); 

if (prenome != NULL){
    l_nome = gtk_label_new(prenome);
    gtk_box_pack_end(GTK_BOX(hbox), l_nome, FALSE, TRUE, 2);
    gtk_widget_show(l_nome);
  }
  


  
  gtk_widget_show(hbox);

  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);  
  return entry;
}

/** Cria um  label e adiciona em box 
 * tambem eh um label para o nome
 */
GtkWidget *new_mostrador(GtkWidget *box,  gchar *nome, gchar *text)
{
  
  GtkWidget *label, *hbox, *l_nome;
  
  hbox = gtk_hbox_new(FALSE, 0);
  label = gtk_label_new(text);
  
  gtk_widget_set_usize(label, 70, -1);
  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, TRUE, 0);
  
  l_nome = gtk_label_new(nome);
  gtk_box_pack_end(GTK_BOX(hbox), l_nome, FALSE, TRUE, 2);
  
  gtk_widget_show(label); 
  gtk_widget_show(l_nome);
  gtk_widget_show(hbox);
  
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
  
  return label;
}


/**
 * esta area esta usando tres eventos: expose, configure e motion
 */
GtkWidget *new_area_desenho(int width, int height, gpointer data)
{
  GtkWidget *drawArea;  

  /*** cria a  drawing area  */
  
  drawArea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawArea), width, height);
  
  gtk_widget_set_events (drawArea, GDK_EXPOSURE_MASK
                         | GDK_LEAVE_NOTIFY_MASK
                         | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_MASK
                         | GDK_POINTER_MOTION_HINT_MASK); 
  
  /* Signals para o pixmapSide de fundo */
  
  /*
    gtk_signal_connect (GTK_OBJECT (drawArea), "expose_event",
                      GTK_SIGNAL_FUNC (expose_event), data);
  gtk_signal_connect (GTK_OBJECT(drawArea),"configure_event",
		      GTK_SIGNAL_FUNC (configure_event), data);
  gtk_signal_connect(GTK_OBJECT(drawArea), "motion_notify_event", 
                     GTK_SIGNAL_FUNC(motion_event), data);
  gtk_signal_connect(GTK_OBJECT(drawArea), "leave_notify_event", 
  		     GTK_SIGNAL_FUNC(leave_event), data);
  */
  /* seta a area de desenho do plano para a area criada */
  return (drawArea);
}




GtkWidget *create_frame(GtkWidget *parent, GtkWidget *conteudo, char *nome)
{
  GtkWidget *frame;
    
  // Cria o frame
  frame = gtk_frame_new(nome);
  gtk_frame_set_shadow_type(GTK_FRAME( frame ), GTK_SHADOW_ETCHED_OUT);
  gtk_container_border_width(GTK_CONTAINER( frame ), 3);
 
  // Adiciona o conteudo (se existir)
  if (conteudo != NULL)
    gtk_container_add(GTK_CONTAINER( frame ), conteudo);
  

  // adiciona o frame em parent (se existir)
  if (parent != NULL) {
    if (GTK_IS_BOX(parent))
      gtk_box_pack_start( GTK_BOX(parent), frame, FALSE, FALSE, 0 );
    else
      gtk_container_add( GTK_CONTAINER(parent), frame );  
  }
  
  gtk_widget_show(frame);
  return (frame);
}

/* Frame widget */

GtkWidget *new_frame(GtkWidget *conteudo, char *nome)
{
  GtkWidget *frame;
    
  /* cria um frame */
  
  frame = gtk_frame_new(nome);
  gtk_frame_set_shadow_type(GTK_FRAME( frame ), GTK_SHADOW_ETCHED_OUT);
  gtk_container_border_width(GTK_CONTAINER( frame ), 3);
  gtk_container_add(GTK_CONTAINER( frame ), conteudo);
  
  gtk_widget_show(conteudo);  
  gtk_widget_show(frame);
  
  return (frame);
}

/* Frame widget */
GtkWidget *
add_frame( GtkWidget *parent, const char *title )
{
  GtkWidget *frame;
  
  frame = gtk_frame_new( title );
  if (parent != NULL) {
    if (GTK_IS_BOX(parent))
      gtk_box_pack_start( GTK_BOX(parent), frame, FALSE, FALSE, 0 );
    else
      gtk_container_add( GTK_CONTAINER(parent), frame );
    gtk_widget_show( frame );
  }
  
  return frame;
}



/* The horizontal value slider widget */
GtkWidget *
add_hscale( GtkWidget *parent_w, GtkAdjustment *adjustment )
{
  GtkWidget *hscale_w;
  
  hscale_w = gtk_hscale_new( GTK_ADJUSTMENT(adjustment) );
  gtk_scale_set_digits( GTK_SCALE(hscale_w), 0 );
  if (GTK_IS_BOX(parent_w))
    gtk_box_pack_start( GTK_BOX(parent_w), hscale_w, FALSE, TRUE, 0 );
  else
    gtk_container_add( GTK_CONTAINER(parent_w), hscale_w );
  gtk_widget_show( hscale_w );
  
  return hscale_w;
}


/* The vertical value slider widget */
GtkWidget *
add_vscale( GtkWidget *parent_w, GtkAdjustment *adjustment )
{
  GtkWidget *vscale_w;
  
  vscale_w = gtk_vscale_new( GTK_ADJUSTMENT(adjustment) );
  gtk_scale_set_value_pos( GTK_SCALE(vscale_w), GTK_POS_RIGHT );
  gtk_scale_set_digits( GTK_SCALE(vscale_w), 0 );
  if (GTK_IS_BOX(parent_w))
    gtk_box_pack_start( GTK_BOX(parent_w), vscale_w, TRUE, TRUE, 0 );
  else
    gtk_container_add( GTK_CONTAINER(parent_w), vscale_w );
  gtk_widget_show( vscale_w );
  
  return vscale_w;
}

GtkObject *
new_adjustment( float value, float low, float upper,
	       void *callback, gpointer callback_data)
{
  GtkObject *adjust;
    
  /* value, lower, upper, step_increment, page_increment, page_size */
  /* Note that the page_size value only makes a difference for
   * scrollbar widgets, and the highest value you'll get is actually
   * (upper - page_size). */
  adjust = gtk_adjustment_new( value, low, upper, 1.0, 1.0, 0.0 );
  
  gtk_signal_connect( GTK_OBJECT(adjust), "value_changed", 
		      GTK_SIGNAL_FUNC( callback ), callback_data );
  
  //  if (GTK_IS_BOX(parent))
  //  gtk_box_pack_start( GTK_BOX(parent), vscale, FALSE, FALSE, 0 );
  //else
  // gtk_container_add( GTK_CONTAINER(frame), vscale );
  

  return (adjust);
}





/******************************
 Funcoes para criacao de Menus
******************************/

/* Cria uma barra de menu */
GtkWidget *new_menu_bar(GtkWidget *parent_box)
{
  GtkWidget *menubar = gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(parent_box), menubar, FALSE, TRUE, 0);
  gtk_widget_show(menubar);
  return menubar;
}

/* adiciona um menu a um menubar */
GtkWidget *new_menu(GtkWidget *menu_bar, char *name)
{

  
  GtkWidget *menu;
  
  menu = gtk_menu_new( );

  if (menu_bar != NULL){
    GtkWidget *menu_item;
    menu_item = gtk_menu_item_new_with_label( name );
    gtk_menu_bar_append( GTK_MENU_BAR(menu_bar), menu_item );
    gtk_widget_show( menu_item );
    gtk_menu_item_set_submenu( GTK_MENU_ITEM(menu_item), menu );
  }
  
  
  return menu;
}

/* adiciona item a um menu, e associa a funcao callback o evento activate */
GtkWidget *new_menu_item(GtkWidget *parent_menu, char *label, void *funcao, 
			 gpointer data)
{
  GtkWidget *menu_item;
  
  menu_item = gtk_menu_item_new_with_label( label );
  gtk_menu_append( GTK_MENU(parent_menu), menu_item);
  
  if (funcao != NULL)
    gtk_signal_connect( GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(funcao), data );
  
  gtk_widget_show( menu_item );
  
  return menu_item;
}

GtkWidget *new_check_menu_item( GtkWidget *menu, const char *label, 
				int init_state, void *funcao, gpointer data )
{
  GtkWidget *checkmenu_item;
  
  checkmenu_item = gtk_check_menu_item_new_with_label( label );
  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(checkmenu_item),
				  init_state );
  gtk_check_menu_item_set_show_toggle( GTK_CHECK_MENU_ITEM(checkmenu_item),TRUE );
  gtk_menu_append( GTK_MENU(menu), checkmenu_item );
  if (funcao != NULL)
    gtk_signal_connect( GTK_OBJECT(checkmenu_item), "toggled",
			GTK_SIGNAL_FUNC(funcao), data );
  gtk_widget_show( checkmenu_item );

  return checkmenu_item;
}



/*??*/
void
new_menu_separator( GtkWidget *parent )
{
  return(add_separator(parent));
  
}

/* Option menu definiton
 * Call this however many times necessary, and then call add_option_menu( )
 * to stick in the finished widget */
GtkWidget *
option_menu_item( const char *label, void *callback, void *callback_data )
{
  GtkWidget *menu_item_w;
  
  menu_item_w = gtk_menu_item_new_with_label( label );
  if (callback != NULL) {
    gtk_signal_connect( GTK_OBJECT(menu_item_w), "activate",
			GTK_SIGNAL_FUNC(callback), callback_data );
  }
  add_option_menu( menu_item_w );
  
  return menu_item_w;
}


/* Add the finished option menu into the specified box widget
 * (widget can only be a menu item when called from option_menu_item( )) */

GtkWidget *
add_option_menu( GtkWidget *widget )
{
  static GtkWidget *menu_w = NULL;
  GtkWidget *optmenu_w;
  
  if (GTK_IS_MENU_ITEM(widget)) {
    if (menu_w == NULL)
      menu_w = gtk_menu_new( );
    gtk_menu_append( GTK_MENU(menu_w), widget );
    gtk_widget_show( widget );
    return NULL;
  }
  
  /* widget is a Gtk?box */
  optmenu_w = gtk_option_menu_new( );
  gtk_option_menu_set_menu( GTK_OPTION_MENU(optmenu_w), menu_w );
  gtk_box_pack_start( GTK_BOX(widget), optmenu_w, FALSE, FALSE, 0 );
  gtk_widget_show( optmenu_w );
  menu_w = NULL;
  
  return optmenu_w;
}


//-----------------------------------------------------------------------
/* Sets up keybindings (accelerators)
 * Call first with widget == NULL, then with any number of widget/keystroke
 * pairs, and finally with the parent window. */

void
keybind( GtkWidget *widget, char *keys )
{
  static GtkAccelGroup *accel_group;
  
  int mods;
  char k;
  
  if (widget == NULL) {
    accel_group = gtk_accel_group_new( );
    return;
  }
  
  if (GTK_IS_WINDOW(widget)) {
    gtk_accel_group_attach( accel_group, GTK_OBJECT(widget) );
    return;
  }
  
  /* Parse keystroke string */
  switch (keys[0]) {
  case '^':
    /* Ctrl- keystroke specified */
    mods = GDK_CONTROL_MASK;
    k = keys[1];
    break;
    
  default:
    /* Simple keypress */
    mods = 0;
    k = keys[0];
    break;
  }
  
  if (GTK_IS_MENU_ITEM(widget)) {
    gtk_widget_add_accelerator( widget, "activate", accel_group, k, mods,
				GTK_ACCEL_VISIBLE );
    return;
  }
  if (GTK_IS_BUTTON(widget)) {
    gtk_widget_add_accelerator( widget, "clicked", accel_group, k, mods,
				GTK_ACCEL_VISIBLE );
    return;
  }
  
  /* Default: make widget grab focus */
  gtk_widget_add_accelerator( widget, "grab_focus", accel_group, k, mods, 
			      GTK_ACCEL_VISIBLE );
}







/* Creates a file selection dialog and show it */
GtkWidget *new_filesel_window(const char *title, const char *init_filename,
			      int show_fileops, void *callback_ok, gpointer data_ok )
{
  GtkWidget *filesel;
  
  filesel = gtk_file_selection_new( title );
  
  if (!show_fileops)
    gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(filesel) );
  
  if (init_filename != NULL)
    gtk_file_selection_set_filename( GTK_FILE_SELECTION(filesel), init_filename );
  
  /* CANCEL ainda nao esta sendo tratado */
  gtk_signal_connect( GTK_OBJECT(filesel), "destroy",
  		      NULL, NULL );
  gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
  		      "clicked", NULL, NULL );


  gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(callback_ok), data_ok );

  gtk_widget_show(filesel);
  return filesel;
}

//------------------------------------------------------------------------
// Creates a color selection window
//------------------------------------------------------------------------
GtkWidget* new_colorsel_window (void *callback_ok)
{
  GtkWidget *colorsel;
  
  colorsel = gtk_color_selection_dialog_new ("Select Color");
  gtk_container_border_width (GTK_CONTAINER (colorsel), 5);
  
  /* CANCEL ainda nao esta sendo tratado */
  gtk_signal_connect( GTK_OBJECT(colorsel), "destroy",
  		      NULL, NULL );
  gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(colorsel)->cancel_button),
  		      "clicked", NULL, NULL );
  
  gtk_signal_connect( GTK_OBJECT(GTK_FILE_SELECTION(colorsel)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(callback_ok), NULL );
  
  return colorsel;
}

/* ATENCAO:  retorna o text area e nao o window */
//------------------------------------------------------------------------
// Creates a color selection window
//------------------------------------------------------------------------
GtkWidget *new_text_window(const char *label)
{
  
  GtkWidget *text_window;
  GtkWidget *text_area;
  
  /* cria janela */
  text_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (text_window), label);
  gtk_window_set_default_size (GTK_WINDOW(text_window),400,400);
  
  /*** cria um vbox na janela inteira  */
  {
    GtkWidget *hbox, *scroll;
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(text_window), hbox);
    
    
    text_area = gtk_text_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(hbox), text_area, TRUE, TRUE, 0);
    gtk_widget_show(text_area);
    
    scroll = gtk_vscrollbar_new(GTK_TEXT(text_area)->vadj);
    gtk_box_pack_start(GTK_BOX(hbox), scroll, FALSE, FALSE, 0);
    gtk_widget_show(scroll);
    
    gtk_widget_show(hbox);
  }  
  
  gtk_widget_show (text_window);
  return text_area;
}





/* Creates a base dialog window */
GtkWidget *
make_dialog_window( const char *title, void *callback_close )
{
  GtkWidget *dialog_window_w;
  
  dialog_window_w = gtk_window_new( GTK_WINDOW_DIALOG );
  gtk_window_set_title( GTK_WINDOW(dialog_window_w), title );
  gtk_container_set_border_width( GTK_CONTAINER(dialog_window_w), 0 );
  if (callback_close != NULL) {
    gtk_signal_connect( GTK_OBJECT(dialog_window_w), "destroy",
			GTK_SIGNAL_FUNC(callback_close), NULL );
  }
  /* Note, no gtk_widget_show( ) */
  
  return dialog_window_w;
}

/* Confirmation dialog */
GtkWidget *
new_ok_cancel_dialog(GtkWidget *conteudo,  const char *title,  
		     void *callback, gpointer data )
     //void *cancel_callback, gpointer cancel_data)
{
  GtkWidget *dialog_win;
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *hbox;
  GtkWidget *button;
  
  dialog_win =  gtk_window_new( GTK_WINDOW_DIALOG );
  gtk_window_set_title( GTK_WINDOW(dialog_win), title );
  //make_dialog_window( title, NULL ); // !!!

  gtk_window_set_position( GTK_WINDOW(dialog_win), GTK_WIN_POS_MOUSE );
  gtk_signal_connect( GTK_OBJECT(dialog_win), "delete_event",
  		      GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      NULL );
  
 
  
  vbox =  add_vbox( dialog_win, FALSE, 10 ); 
 
  // adiciona o conteudo no frame 
  gtk_box_pack_start( GTK_BOX(vbox), conteudo,TRUE,TRUE,0 );
 
  frame = add_frame( vbox, NULL ); 
  // cria o box para os dois botoes (OK e Cancel)
  hbox = add_hbox(frame, FALSE, 10 );
  
  // OK button 
  button = gtk_button_new_with_label("OK");
  gtk_box_pack_start( GTK_BOX(hbox), button, TRUE, TRUE, 0 );
  
  gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
			     GTK_SIGNAL_FUNC(gtk_widget_hide), 
			     GTK_OBJECT(dialog_win) );
  gtk_signal_connect( GTK_OBJECT(button), "clicked",
		      GTK_SIGNAL_FUNC(callback), data );
  gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			     GTK_OBJECT(dialog_win) );
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  
  gtk_widget_show( button );
  
  add_hbox( hbox, TRUE, 0 ); /* spacer */
  
  // Cancel button 
  button = gtk_button_new_with_label("Cancel");
  
  gtk_box_pack_start( GTK_BOX(hbox), button, TRUE, TRUE, 0 );
  //gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
  //			     GTK_SIGNAL_FUNC(gtk_widget_hide), 
  //			     GTK_OBJECT(dialog_win) );
  //gtk_signal_connect( GTK_OBJECT(button), "clicked",
  //GTK_SIGNAL_FUNC(gtk_widget_destroy), cancel_data );
  gtk_signal_connect_object( GTK_OBJECT(button), "clicked",
  			     GTK_SIGNAL_FUNC(gtk_widget_destroy), 
  			     GTK_OBJECT(dialog_win));
  gtk_widget_show( button );
  gtk_widget_show( dialog_win );
  
  return dialog_win;
}

/* Creates and installs an OpenGL widget into specified box */
GtkWidget *
add_gl_area( GtkWidget *parent )
{
  GtkWidget *glarea;

  int gl_area_attributes[] = {
    GDK_GL_RGBA,
    GDK_GL_RED_SIZE, 1,
    GDK_GL_GREEN_SIZE, 1,
    GDK_GL_BLUE_SIZE, 1,
    GDK_GL_DEPTH_SIZE, 1,
    GDK_GL_DOUBLEBUFFER,
    GDK_GL_NONE
  };
  
  glarea = gtk_gl_area_new( gl_area_attributes );
  
  if ( GTK_IS_BOX(parent) )
    gtk_box_pack_start( GTK_BOX(parent), glarea, TRUE, TRUE, 0 );
  else
    gtk_container_add( GTK_CONTAINER(parent), glarea );


  gtk_widget_show( glarea );
  gtk_widget_set_events(GTK_WIDGET(glarea), 
			GDK_EXPOSURE_MASK           |
			GDK_BUTTON_PRESS_MASK       |
			GDK_BUTTON_RELEASE_MASK     |
			GDK_POINTER_MOTION_MASK     |
			GDK_POINTER_MOTION_HINT_MASK|
			GDK_BUTTON_MOTION_MASK      |
			GDK_BUTTON1_MOTION_MASK     |
			GDK_BUTTON2_MOTION_MASK     | 
			GDK_BUTTON3_MOTION_MASK);

  /* Attach the signal handlers */
  //gtk_signal_connect( GTK_OBJECT(glarea), "realize",
  //		      GTK_SIGNAL_FUNC(gl_initialize), NULL );
  //gtk_signal_connect( GTK_OBJECT(glarea), "expose_event",
  //		      GTK_SIGNAL_FUNC(gl_refresh), NULL );
  //gtk_signal_connect( GTK_OBJECT(glarea), "configure_event",
  //		      GTK_SIGNAL_FUNC(gl_resize), NULL );
  // gtk_signal_connect( GTK_OBJECT(glarea), "button_press_event",
  //		      GTK_SIGNAL_FUNC(camera_move), NULL );
  // gtk_signal_connect( GTK_OBJECT(glarea), "button_release_event",
  //		      GTK_SIGNAL_FUNC(camera_move), NULL );
  // gtk_signal_connect( GTK_OBJECT(glarea), "motion_notify_event",
  //		      GTK_SIGNAL_FUNC(camera_move), NULL );
  
  return glarea;
}



/*funcao auxiliar,  apenas insere um gfloat no texto do entry ! */
void  entry_set_float(GtkWidget *entry, gfloat v, char *s)
{
  GString *text = g_string_new(NULL);
  g_string_sprintf(text,s,v);
  gtk_entry_set_text(GTK_ENTRY(entry),text->str);
  g_string_free(text,TRUE);
}

/*funcao auxiliar,  apenas insere um gfloat no texto do entry ! */
void  entry_set_long(GtkWidget *entry, long v)
{
  GString *text = g_string_new(NULL);
  g_string_sprintf(text,"%ld",v);
  gtk_entry_set_text(GTK_ENTRY(entry),text->str);
  g_string_free(text,TRUE);
}

/*funcao auxiliar,  apenas insere um gfloat no texto do label ! */
void  label_set_float(GtkWidget *label, gfloat v, char *s)
{
  GString *text = g_string_new(NULL);
  g_string_sprintf(text,s,v);
  gtk_label_set_text(GTK_LABEL(label),text->str);
  g_string_free(text,TRUE);
}
/*funcao auxiliar,  apenas insere um gfloat no texto do label ! */
void  label_set_long(GtkWidget *label, long v)
{
  GString *text = g_string_new(NULL);
  g_string_sprintf(text,"%ld",v);
  gtk_label_set_text(GTK_LABEL(label),text->str);
  g_string_free(text,TRUE);
}

char *entry_get_text(GtkWidget *widget)
{
  return (gtk_entry_get_text(GTK_ENTRY(widget )));
}


float entry_get_float(GtkWidget *widget)
{
  return ( atof(gtk_entry_get_text(GTK_ENTRY(widget))));
}















